/*********************************************************************************
 * Includes
 ********************************************************************************/
// Basically, first, as on top of all HAM files, the most
// important include is the mygba.h, which contains all the
// macro definitions, and the HAMlib function prototypes.
#include <mygba.h>
// Include the HEL Library Header
// You can download HEL at http://www.console-dev.de
#include <hel.h>


/*********************************************************************************
 * Globals
 ********************************************************************************/
u32 g_VblCounter=0;
u32 g_HblCounter=0;

/*********************************************************************************
 * Prototypes
 ********************************************************************************/
void vblFunc(void);
void hblFunc(void);

/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    // Initialize HAMlib
    // Then we call the single most important function of ham, ham_Init .
    // This function must be called first thing when working with HAMlib,
    // because all ham_* functions will only work correctly after this has been called.
    // Behaviour of HAMlib is undefined if you comment this out.
    ham_Init();

    ham_SetBgMode(0);
    ham_InitText(0);

    ham_StartIntHandler(INT_TYPE_VBL, (void*)vblFunc); // VBL
    ham_StartIntHandler(INT_TYPE_HBL, (void*)hblFunc); // HBL
    
    // Stop Interrupts again
    // comment out to test
    hel_IntrStopHandler(INT_TYPE_VBL);
    //hel_IntrStopHandler(INT_TYPE_HBL);
    
    // Loop
    while(TRUE)
    {
         // simply dump the countervalue so we can
         // check if the interrupt still gets triggered
         ham_DrawText(1,1,"VBL: %d", g_VblCounter);
         ham_DrawText(1,2,"HBL: %d", g_HblCounter);
    }

    return 0;
}


void vblFunc(void)
{
    g_VblCounter++;
}


void hblFunc(void)
{
    g_HblCounter++;
}

/* END OF FILE */
